#include "defs.h"

void spiInit() {

    // Define the following pin as input
    DDRB &= ~(1<<USI_DI); // 0 - input; 1 - output

    // Define the following pins as output
    DDRB |= ((1<<USI_DO) | (1<<USI_SCK) | (1<<USI_CS));

    // Three wire mode, Datasheet, "USI Control Register  USICR" tab
    USICR &= ~(1<<USIWM1);
    USICR |= (1<<USIWM0);

    // CS high
    PORTB |= ~(0<<USI_CS);

    return;
}//spiInit

unsigned int shft16bit(unsigned int val) {
unsigned char higherByte, lowerByte;
unsigned int res;
const unsigned char CR = (1<<USIWM0) | (1<<USICS1) | (1<<USICLK) | (1<<USITC);

      higherByte = (unsigned char)((val & 0xFF00) >> 8);
      lowerByte  = (unsigned char)(val & 0x00FF);

      PORTB &= (0<<USI_CS); // CS low

      USIDR = higherByte;
      USISR |= (1<<USIOIF);

      do {
         USICR = CR;
      } while ((USISR & (1<<USIOIF)) == 0);

      res = USIDR << 8;
//      res = USIDR;
//      res <<= 8;

      USIDR = lowerByte;
      USISR |= (1<<USIOIF);

      do {
         USICR = CR;
      } while ((USISR & (1<<USIOIF)) == 0);

      res |= USIDR;

      PORTB |= ~(0<<USI_CS); // CS high

      return res;
}//shft16bit

// https://github.com/janelia-arduino/AD7328/blob/master/src/AD7328.cpp
void setRangeRegister(unsigned char ch, unsigned char range) {
unsigned int range_ = 0xA000;

     if ((ch < 0) || (ch > 7)) return;

     switch (ch) {
       case 0:
         range_ &= CH0_MASK;
         range_ |= range << 11;
       break;
       case 1:
         range_ &= CH1_MASK;
         range_ |= range << 9;
       break;
       case 2:
         range_ &= CH2_MASK;
         range_ |= range << 7;
       break;
       case 3:
         range_ &= CH3_MASK;
         range_ |= range << 5;
       break;
       case 4:
         range_ = range & CH4_MASK;
         range_ |= range << 11;
       break;
       case 5:
         range_ = range & CH5_MASK;
         range_ |= range << 9;
       break;
       case 6:
         range_ = range & CH6_MASK;
         range_ |= range << 7;
       break;
       case 7:
         range_ = range & CH7_MASK;
         range_ |= range << 5;
       break;
       default: break;
     }//switch

     shft16bit(range_);

     return;
}//setRangeRegister

void setControlRegister(unsigned char ch, unsigned int setup) {

     if ((ch < 0) || (ch > 7)) return;

     setup |= ((ch & 0x07) << 10);

     shft16bit(setup);

     return;
}//setControlRegister

void ADC7328init(void) {
unsigned char i = 0;

     for (i = 0; i < 7; i++) setRangeRegister(i, BIP10); // Fill in both range registers
     setControlRegister(0, SETUP_1);                     // Control register
     shft16bit(0xFE00);                                  // Sequence register (no kidding)

     return;
}//ADC7328init

void main() {
unsigned int result = 0;
char buffer[8];

     spiInit();
     ADC7328init();
     // Datasheet, "System Clock Prescalar" tab
     CLKPR = 0x80;
     CLKPR = 0x00;
     UART1_Init_Advanced(9600, _UART_NOPARITY, _UART_ONE_STOPBIT);
     Delay_ms(100);

     while (1) {
        result = shft16bit(0x0000) & 0x1FFF; // solely gets least 13 bits
        if ((result & 0x1000) >> 12) // bit12 = 1
           IntToStr(result & 0x0FFF, buffer);
        else
           IntToStr((~result) & 0x0FFF, buffer);
        UART1_Write_Text(buffer);
        nl();
        Delay_ms(250);
     }//while

     return;
}//main